@@ -1065,7 +1065,7 @@ class Scanner(object):
             ch = srp(length)
         if not length:
             raise ScannerError(
-                'while scanning an %s' % name,
+                'while scanning an %s' % (name,),
                 start_mark,
                 'expected alphabetic or numeric character, but found %r' % utf8(ch),
                 self.reader.get_mark(),
@@ -1077,7 +1077,7 @@ class Scanner(object):
         # assert ch1 == ch
         if ch not in '\0 \t\r\n\x85\u2028\u2029?:,[]{}%@`':
             raise ScannerError(
-                'while scanning an %s' % name,
+                'while scanning an %s' % (name,),
                 start_mark,
                 'expected alphabetic or numeric character, but found %r' % utf8(ch),
                 self.reader.get_mark(),
@@ -1237,7 +1237,7 @@ class Scanner(object):
             # Eat whitespaces and comments until we reach the next token.
             comment = self.scan_to_next_token()
             while comment:
-                trailing.append(comment[0])
+                trailing.append(' ' * comment[1].column + comment[0])
                 comment = self.scan_to_next_token()
 
             # Keep track of the trailing whitespace and following comments
@@ -1638,7 +1638,7 @@ class Scanner(object):
         ch = srp()
         if ch != '!':
             raise ScannerError(
-                'while scanning a %s' % name,
+                'while scanning a %s' % (name,),
                 start_mark,
                 "expected '!', but found %r" % utf8(ch),
                 self.reader.get_mark(),
@@ -1652,7 +1652,7 @@ class Scanner(object):
             if ch != '!':
                 self.reader.forward(length)
                 raise ScannerError(
-                    'while scanning a %s' % name,
+                    'while scanning a %s' % (name,),
                     start_mark,
                     "expected '!', but found %r" % utf8(ch),
                     self.reader.get_mark(),
@@ -1675,6 +1675,7 @@ class Scanner(object):
             or 'A' <= ch <= 'Z'
             or 'a' <= ch <= 'z'
             or ch in "-;/?:@&=+$,_.!~*'()[]%"
+            or ((self.scanner_processing_version > (1, 1)) and ch == "#")
         ):
             if ch == '%':
                 chunks.append(self.reader.prefix(length))
@@ -1690,7 +1691,7 @@ class Scanner(object):
             length = 0
         if not chunks:
             raise ScannerError(
-                'while parsing a %s' % name,
+                'while parsing a %s' % (name,),
                 start_mark,
                 'expected URI, but found %r' % utf8(ch),
                 self.reader.get_mark(),
@@ -1709,7 +1710,7 @@ class Scanner(object):
             for k in range(2):
                 if srp(k) not in '0123456789ABCDEFabcdef':
                     raise ScannerError(
-                        'while scanning a %s' % name,
+                        'while scanning a %s' % (name,),
                         start_mark,
                         'expected URI escape sequence of 2 hexdecimal numbers,'
                         ' but found %r' % utf8(srp(k)),
@@ -1726,7 +1727,7 @@ class Scanner(object):
             else:
                 value = unicode(b"".join(code_bytes), 'utf-8')
         except UnicodeDecodeError as exc:
-            raise ScannerError('while scanning a %s' % name, start_mark, str(exc), mark)
+            raise ScannerError('while scanning a %s' % (name,), start_mark, str(exc), mark)
         return value
 
     def scan_line_break(self):
@@ -1794,7 +1795,7 @@ class RoundTripScanner(Scanner):
             if isinstance(self.tokens[0], CommentToken):
                 self.tokens_taken += 1
                 comment = self.tokens.pop(0)
-                # print 'dropping2', comment
+                # nprint('dropping2', comment)
                 comments.append(comment)
         if len(comments) >= 1:
             self.tokens[0].add_pre_comments(comments)
@@ -1824,7 +1825,34 @@ class RoundTripScanner(Scanner):
                 and self.tokens[0].end_mark.line == self.tokens[1].start_mark.line
             ):
                 self.tokens_taken += 1
-                self.tokens[0].add_post_comment(self.tokens.pop(1))
+                c = self.tokens.pop(1)
+                self.fetch_more_tokens()
+                while len(self.tokens) > 1 and isinstance(self.tokens[1], CommentToken):
+                    self.tokens_taken += 1
+                    c1 = self.tokens.pop(1)
+                    c.value = c.value + (' ' * c1.start_mark.column) + c1.value
+                    self.fetch_more_tokens()
+                self.tokens[0].add_post_comment(c)
+            elif (
+                len(self.tokens) > 1
+                and isinstance(self.tokens[0], ScalarToken)
+                and isinstance(self.tokens[1], CommentToken)
+                and self.tokens[0].end_mark.line != self.tokens[1].start_mark.line
+            ):
+                self.tokens_taken += 1
+                c = self.tokens.pop(1)
+                c.value = (
+                    '\n' * (c.start_mark.line - self.tokens[0].end_mark.line)
+                    + (' ' * c.start_mark.column)
+                    + c.value
+                )
+                self.tokens[0].add_post_comment(c)
+                self.fetch_more_tokens()
+                while len(self.tokens) > 1 and isinstance(self.tokens[1], CommentToken):
+                    self.tokens_taken += 1
+                    c1 = self.tokens.pop(1)
+                    c.value = c.value + (' ' * c1.start_mark.column) + c1.value
+                    self.fetch_more_tokens()
             self.tokens_taken += 1
             return self.tokens.pop(0)
         return None
@@ -1877,6 +1905,10 @@ class RoundTripScanner(Scanner):
                 while ch not in _THE_END:
                     ch = srp()
                     if ch == '\0':  # don't gobble the end-of-stream character
+                        # but add an explicit newline as "YAML processors should terminate
+                        # the stream with an explicit line break
+                        # https://yaml.org/spec/1.2/spec.html#id2780069
+                        comment += '\n'
                         break
                     comment += ch
                     srf()
